// Z-Viewer, written by Zenja Solaja, December 2000
// Z-Viewer is a zoomable image viewer.

#include <InterfaceKit.h>
#include <StorageKit.h>
#include <TranslationKit.h>

#include "ZWindow.h"
#include "ZView.h"
#include "main.h"
#include "linked_list.h"
#include "ZDisplayLibs.h"
#include "ZScrollView.h"

//==============================================
ZWindow::ZWindow(BRect screenbounds)
	:BWindow(screenbounds, "Z-Viewer 0.94",B_TITLED_WINDOW, 0)
{
	screenbounds.OffsetTo(B_ORIGIN);
	BRect menuBarRect = screenbounds;
	menuBarRect.bottom = MENU_BAR_HEIGHT;

	PrepareMenu(menuBarRect);	
	AddChild(fmenuBar);
		
	fimage_title = NULL;
			
	screenbounds.top += MENU_BAR_HEIGHT; //Leave space for Taskbar
	zv_backview = new ZBackgroundView(screenbounds);
	AddChild(zv_backview);
	
	//PrepareBitmap("startup.jpg");
	Show();
}

//=================================================
void ZWindow::PrepareMenu(BRect menuRect)
{
	// File menu
	fmenuBar = new BMenuBar(menuRect, "MenuBar");
	BMenu *File_menu = new BMenu("File");
	BMenuItem *item;
	File_menu->AddItem(item = new BMenuItem("Select Directory", new BMessage(SELECT_DIR_MSG), 'D'));
	item->SetTarget(be_app);
	File_menu->AddSeparatorItem();
	File_menu->AddItem(item = new BMenuItem("About", new BMessage(ABOUT_MSG), 'A'));
	File_menu->AddItem(item = new BMenuItem("Quit", new BMessage(B_CLOSE_REQUESTED),'Q'));
	
	// Image menu
	fimage_menu = new BMenu("Image");
	fimage_menu->AddItem(item = new BMenuItem("Next Image", new BMessage(NEXT_IMAGE_MSG)));
	fimage_menu->AddItem(item = new BMenuItem("Previous Image", new BMessage(PREV_IMAGE_MSG)));
	fimage_menu->AddSeparatorItem();
	fimage_menu->AddItem(item = new BMenuItem("First Image in List", new BMessage(FIRST_IMAGE_MSG)));
	fimage_menu->AddItem(item = new BMenuItem("Last Image in List", new BMessage(LAST_IMAGE_MSG)));
	fimage_menu->AddSeparatorItem();
	fimage_menu->AddItem(item = new BMenuItem("Zoom In", new BMessage(ZOOM_IN_MSG)));
	fimage_menu->AddItem(item = new BMenuItem("Zoom Out", new BMessage(ZOOM_OUT_MSG)));
	fimage_menu->AddItem(item = new BMenuItem("Full Screen", new BMessage(ZOOM_FULL_MSG),'F'));
		
	fmenuBar->AddItem(File_menu);
}
		
//=================================================
void ZWindow::MessageReceived(BMessage *message)
{
	switch(message->what){
		case ABOUT_MSG: {
			BAlert *alert = new BAlert("",
					"Z-Viewer v0.94, written by Zenja Solaja, Dec 2000.\n\
					\nKeyboard shortcuts (numpad also works):
					\nPage Down		-	Next image in list\
					\nPage Up		-	Previous image in list\
					\nHome			-	First image in list\
					\nEnd			-	Last image in list\
					\n+			-	Zoom In\
					\n-			-	Zoom Out\
					\n*			-	Zoom To Fit\
					\nCursor keys	-	Pan Zoomed Image\
					\nEnter			-	Center Image\
					\nESC			-	Quit", "All Right!!!", NULL, NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT);
					
			alert->Go(); 
			break;
		}
		case NEXT_IMAGE_MSG: {
			if (list_navigation->position == list_navigation->tail)
				list_navigation->position = list_navigation->head;
			else
				list_navigation->position = list_navigation->position->next;
			PrepareBitmap(list_navigation->position->contents);
			break;
		}
		case PREV_IMAGE_MSG: {
			if (list_navigation->position == list_navigation->head)
				list_navigation->position = list_navigation->tail;
			else
				list_navigation->position = list_navigation->position->prev;
			PrepareBitmap(list_navigation->position->contents);
			break;
		}
		case FIRST_IMAGE_MSG: {
			list_navigation->position = list_navigation->head;
			PrepareBitmap(list_navigation->position->contents);
			break;
		}
		case LAST_IMAGE_MSG: {
			list_navigation->position = list_navigation->tail;
			PrepareBitmap(list_navigation->position->contents);
			break;
		}
		case ZOOM_IN_MSG:
		{
			bitmap_view->AnimatedZoom(ZOOM_IN);
			break;
		}
		case ZOOM_OUT_MSG:
		{
			bitmap_view->AnimatedZoom(ZOOM_OUT);
			break;
		}
		case ZOOM_FULL_MSG: {
			fullscreen = !fullscreen;
			BRect aRect;
			
			if (fullscreen)	{
				ZScreenBounds(&aRect);
				zv_window->MoveTo(0,0);
			}
			else {
				aRect.Set(100,100,800,600);
				zv_window->MoveTo(100,100);
			}
			zv_window->ResizeTo(aRect.Width(), aRect.Height());
			bitmap_view->ResizeView();
			bitmap_view->Zoom();
			break;
		}
		default:
			BWindow::MessageReceived(message);
	}
}

//===============================================
bool ZWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return (true);
}

//====================================================
void ZWindow::FrameResized(float width, float height)
{
	bitmap_view->ResizeView();
	bitmap_view->Zoom();
}

//===============================================
void ZWindow::PrepareBitmap(char *filename)
{
	BitmapView *old_view = bitmap_view;
	
	BBitmap *bitmap_image = BTranslationUtils::GetBitmapFile(filename,NULL);
	BRect bRect = bitmap_image->Bounds();
				
	BitmapView *new_bitmap_view = new BitmapView(bitmap_image, bRect, "Bitmap_Image");
	new_bitmap_view->ResizeView();
	zv_window->AddChild(new_bitmap_view);
	bitmap_view = new_bitmap_view;
			
	if (old_view)
	{
		zv_window->RemoveChild(old_view);
		delete old_view;
	}
	else // First image ever shown
	{
		fmenuBar->AddItem(fimage_menu); 
		zv_listwindow->Show();
	}
	zv_listwindow->UpdateSelection();
	PrepareTitle(filename); //add title
}

//===============================================
void ZWindow::PrepareTitle(char *filename)
{
	BPath thePath(filename);
	
	if (fimage_title)
	{
		fmenuBar->RemoveItem(fimage_title);
		delete fimage_title;
	}
	fimage_title = new BMenu(thePath.Leaf());
	fmenuBar->AddItem(fimage_title);
}