// Z-Viewer, written by Zenja Solaja, December 2000
// Z-Viewer is a zoomable image viewer.

#include <AppKit.h>
#include <StorageKit.h>
#include <string.h>

#include "main.h"
#include "ZWindow.h" 
#include "linked_list.h"
#include "ZScrollView.h"

// now what would an app be without a global or two?
LIST_NAVIGATION *list_navigation;
BitmapView		*bitmap_view;
ZWindow			*zv_window;
ZBackgroundView *zv_backview;
bool 			fullscreen; 


//===================================
int main(void)
{
	bitmap_view = NULL;
	fullscreen = false;
	ZViewerApp *zv = new ZViewerApp;
		
	list_navigation = new LIST_NAVIGATION;
	list_navigation->head = list_navigation->tail = list_navigation-> position = NULL;
	
	be_app->PostMessage(SELECT_DIR_MSG);
	zv->Run();
		
	delete_list();
	delete list_navigation;
	delete zv;
}

//======================
ZViewerApp::ZViewerApp()
	:BApplication("application/x-vnd.ZS-ZViewer")
{
	BRect aRect(250, 50, 600, 400);
	zv_window = new ZWindow(aRect);
	zv_window->Show();
	filepanel = new BFilePanel(B_OPEN_PANEL, NULL, NULL, B_DIRECTORY_NODE, false, NULL, NULL, false, true);
	
	aRect.Set(100, 100, 245, 400);
	zv_listwindow = new ZListWindow(aRect);
}

//===========================
void ZViewerApp::MessageReceived(BMessage *message)
{
	switch(message->what){
		case SELECT_DIR_MSG: {
			BWindow *temp = filepanel->Window();
			temp->SetTitle("Select Directory with Graphic Images");
			filepanel->Show();
			break;
		}
		default:
			BApplication::MessageReceived(message);
	}
}

//==============================================
void ZViewerApp::RefsReceived(BMessage *message)
{
	uint32 type; 
	int32 count;
	entry_ref ref;
	BPath path;

	message->GetInfo("refs", &type, &count); 
	if ( type != B_REF_TYPE ) return; //sanity check
	if ( message->FindRef("refs", 0, &ref) == B_OK ) { 
		BEntry entry(&ref, true);
		if ( entry.IsDirectory() && entry.GetPath(&path)==B_OK)
		{
			if (drill_through_directory(path.Path()))
				zv_window->PrepareBitmap(list_navigation->position->contents);
			else
			{
				BAlert *alert = new BAlert("", "Search here, search there.\
												Hard disk search everywhere.\
												No images found.", "Oops");
				alert->Go();
			}
		}
	}
}

//============================================
bool drill_through_directory(const char *path) 
{ 
	BDirectory dir(path);
	status_t err = B_OK;
	char *tstr = new char[B_MIME_TYPE_LENGTH];
	BPath workpath;
	BEntry item;
	bool image_found = false;
		
	if (dir.InitCheck() == B_OK) {
		while(dir.GetNextEntry(&item) == B_OK)
		{ 
			item.GetPath(&workpath);
			if (!item.IsDirectory())
			{
				BNode *node = new BNode(workpath.Path());
				if((err = node->InitCheck())==B_OK)
				{
					BNodeInfo *ninfo = new BNodeInfo(node);
					if((err = ninfo->InitCheck())==B_OK)
					{
						if((err = ninfo->GetType(tstr))==B_OK)
						{
							if(!strncmp(tstr,"image/",6)) {
								if (!image_found) { // only delete list if first image
									delete_list();
									zv_listwindow->EmptyList();
									image_found = true;
								}
								add_to_list(workpath.Path()); //Store in linked list
								zv_listwindow->AddItem(list_navigation->tail->bstring); 
							}
						}
					}
					delete ninfo;
				}
				delete node; 
			}
		}
	}
	delete tstr;
	if (image_found)
		list_navigation->position = list_navigation->head;  //start from top
	return (image_found);
} 
